@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009-2012  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2013  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,13 +20,14 @@
 # Original filename: widget-set.c
 # Original copyright and license: 2003, 2004  John Darrington, GPL3+
 
-from __future__ import print_function, division, unicode_literals
+
 
 from collections import OrderedDict
 
 # pylint: disable=W0614,W0401
-from PySide.QtCore import *
-from PySide.QtGui import *
+from PyQt4.QtCore import *
+from PyQt4.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
+from PyQt4.QtGui import *
 
 from .debug import *
 # pylint: enable=W0614,W0401
@@ -37,7 +38,6 @@ from .model import models
 from . import gamestate
 from . import drawingarea
 from .settings import settings
-from .ui.main import Ui_MainWindow
 
 try:
     _, ngettext
@@ -48,7 +48,7 @@ except NameError:
     
     
 class AnimType (object): # pylint: disable=R0903
-    NONE, FWD, BWD, KEY, NEW = range(5)
+    NONE, FWD, BWD, KEY, NEW = list(range(5))
     
     
 class ElideLabel (QLabel):
@@ -67,19 +67,40 @@ class ElideLabel (QLabel):
         p.drawText(self.rect(), Qt.TextSingleLine, self.text())
         
         
+class QMoveEdit (QLineEdit):
+    nextword = Signal()
+    prevword = Signal()
+    swapnext = Signal()
+    swapprev = Signal()
+    
+    def keyPressEvent(self, event):
+        if event.matches(QKeySequence.MoveToNextWord):
+            self.nextword.emit()
+        elif event.matches(QKeySequence.MoveToPreviousWord):
+            self.prevword.emit()
+        elif event.key() == Qt.Key_Right and event.modifiers() == Qt.AltModifier:
+            self.swapnext.emit()
+        elif event.key() == Qt.Key_Left and event.modifiers() == Qt.AltModifier:
+            self.swapprev.emit()
+        else:
+            QLineEdit.keyPressEvent(self, event)
+            
+        
 class MainWindow (QMainWindow):
     
     ### init ###
     
-    def __init__(self):
+    def __init__(self, opts):
         QMainWindow.__init__(self)
+        self.ui = dialogs.UI(self, 'main.ui')
+        from .ui.main import retranslate
+        retranslate(self)
         
         # UI
-        self.ui = Ui_MainWindow()
-        self.ui.setupUi(self)
         self.ui.splitter.setCollapsible(0, False)
         self.ui.splitter.setStretchFactor(0, 1)
         self.ui.splitter.setStretchFactor(1, 0)
+        self.ui.label_debug_text.setVisible(DEBUG_DRAW or DEBUG_FPS)
         
         # set icons
         self.setWindowIcon(QIcon(config.IMAGE_FILE))
@@ -110,21 +131,22 @@ class MainWindow (QMainWindow):
         
         # widgets that are not created with Qt Designer
         self.playbarstate = self.create_toolbar()
-        self.cube_area = self.create_cube_area()
+        self.edit_moves = QMoveEdit()
+        self.edit_moves.setObjectName("edit_moves")
+        self.edit_moves.setFrame(False)
+        self.ui.layout_moves.insertWidget(1, self.edit_moves)
+        self.edit_moves.returnPressed.connect(self.on_edit_moves_returnPressed)
+        self.edit_moves.nextword.connect(self.on_edit_moves_nextword)
+        self.edit_moves.prevword.connect(self.on_edit_moves_prevword)
+        self.edit_moves.swapprev.connect(self.on_edit_moves_swapprev)
+        self.edit_moves.swapnext.connect(self.on_edit_moves_swapnext)
+        self.cube_area = self.create_cube_area(opts)
         self.cube_area.drop_color.connect(self.on_cubearea_drop_color)
         self.cube_area.drop_file.connect(self.on_cubearea_drop_file)
-        self.setTabOrder(self.ui.edit_moves, self.cube_area)
+        self.cube_area.debug_text.connect(self.on_cubearea_debug_text)
+        self.setTabOrder(self.edit_moves, self.cube_area)
         self.setTabOrder(self.cube_area, self.ui.box_sidepane)
         self.cube_area.setFocus(Qt.OtherFocusReason)
-        # actions that belongs to no widget
-        self.action_jump_to_editbar = QAction(self)
-        self.action_jump_to_editbar.setShortcut(QKeySequence(settings.action.edit_moves))
-        self.action_jump_to_editbar.triggered.connect(self.on_action_jump_to_editbar_triggered)
-        self.addAction(self.action_jump_to_editbar)
-        self.action_edit_model = QAction(self)
-        self.action_edit_model.setShortcut(QKeySequence(settings.action.edit_model))
-        self.action_edit_model.triggered.connect(self.on_action_edit_model_triggered)
-        self.addAction(self.action_edit_model)
         self.status_text = ElideLabel()
         p = self.status_text.sizePolicy()
         p.setHorizontalStretch(1)
@@ -134,6 +156,23 @@ class MainWindow (QMainWindow):
         # created when needed
         self.progress_dialog = None
         
+        # actions that belongs to no widget
+        self.action_jump_to_editbar = QAction(self)
+        self.action_jump_to_editbar.setObjectName('action_jump_to_editbar')
+        self.action_jump_to_editbar.setShortcut(QKeySequence(settings.action.edit_moves))
+        self.action_jump_to_editbar.triggered.connect(self.on_action_jump_to_editbar_triggered)
+        self.addAction(self.action_jump_to_editbar)
+        self.action_edit_model = QAction(self)
+        self.action_edit_model.setObjectName('action_edit_model')
+        self.action_edit_model.setShortcut(QKeySequence(settings.action.edit_model))
+        self.action_edit_model.triggered.connect(self.on_action_edit_model_triggered)
+        self.addAction(self.action_edit_model)
+        self.action_reload_shader = QAction(self)
+        self.action_reload_shader.setObjectName('action_reload_shader')
+        self.action_reload_shader.setShortcut(QKeySequence(settings.action.reload_shader))
+        self.action_reload_shader.triggered.connect(self.on_action_reload_shader_triggered)
+        self.addAction(self.action_reload_shader)
+        
         # plugins
         self.treestore = QStandardItemModel()
         self.selected_tree_path = None
@@ -168,20 +207,24 @@ class MainWindow (QMainWindow):
         sizes = self.ui.splitter.sizes()
         self.ui.splitter.setSizes([divider_position, sum(sizes)-divider_position])
         
-    def create_cube_area(self):
-        cube_area = drawingarea.CubeArea()
+    def create_cube_area(self, opts):
+        if DEBUG_THREAD:
+            cube_area = drawingarea.CubeAreaThreaded(opts)
+        else:
+            cube_area = drawingarea.CubeArea(opts)
+        cube_area.setObjectName('drawingarea')
         sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         sizePolicy.setHorizontalStretch(0)
         sizePolicy.setVerticalStretch(0)
         cube_area.setSizePolicy(sizePolicy)
         self.ui.verticalLayout.addWidget(cube_area)
-        cube_area.end_animation.connect(self.on_cubearea_end_animation)
+        cube_area.animation_ending.connect(self.on_cubearea_animation_ending)
         cube_area.request_rotation.connect(self.on_cubearea_request_rotation)
         cube_area.request_swap_blocks.connect(self.on_cubearea_request_swap_blocks)
         cube_area.request_rotate_block.connect(self.on_cubearea_request_rotate_block)
         glformat = cube_area.format()
-        dialogs.PreferencesDialog.accum_buffers = glformat.accum()
         dialogs.PreferencesDialog.sample_buffers = max(glformat.samples(), 1)
+        cube_area.start()
         return cube_area
         
     def create_toolbar(self):
@@ -258,9 +301,7 @@ class MainWindow (QMainWindow):
         mirror_distance = settings.draw.mirror_distance if settings.draw.mirror_faces else None
         model = Model(size, mirror_distance)
         self.gamestate.set_state(model, blocks, settings.game.moves, settings.game.position)
-        self.cube_area.set_model(model)
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
-        self.cube_area.update()
+        self.cube_area.set_model(model, self.gamestate.current_cube_state.blocks)
         self.update_ui()
         
     def new_game(self, mtype=None, size=None, solved=False):
@@ -278,13 +319,11 @@ class MainWindow (QMainWindow):
         model = Model(size, mirror_distance)
         self.gamestate.new_game(model, solved)
         self.unsolved = not solved
-        self.cube_area.set_model(model)
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
-        self.cube_area.update()
+        self.cube_area.set_model(model, self.gamestate.current_cube_state.blocks)
         self.update_ui()
         
     def is_animating(self):
-        return self.cube_area.timer_animate.isActive()
+        return self.cube_area.animation_active
         
     def start_animation(self, move_data, stop_after, animtype):
         if move_data:
@@ -308,7 +347,7 @@ class MainWindow (QMainWindow):
             total = self.gamestate.all_moves.queue_length
             solved = self.gamestate.current_cube_state.is_solved()
             
-            model_text = unicode(self.gamestate.current_cube_state.model)
+            model_text = str(self.gamestate.current_cube_state.model)
             # substitution for {move_text} in statusbar text
             move_text = ngettext("{current} / {total} move",
                                  "{current} / {total} moves",
@@ -389,13 +428,15 @@ class MainWindow (QMainWindow):
     def closeEvent(self, event):
         try:
             dialogs.Dialog.delete()
+            self.cube_area.stop()
             model, blocks, moves, position = self.gamestate.get_state()
             try:
                 mtype = models.index(model.__class__)
             except ValueError:
                 print('unknown model')
             else:
-                settings.keystore.changed.disconnect(self.cube_area.on_settings_changed)
+                settings.keystore.changed.disconnect(self.on_settings_changed)
+                settings.keystore.error.disconnect(self.on_settings_error)
                 settings.game.blocks = blocks
                 settings.game.moves = moves
                 settings.game.position = position
@@ -406,8 +447,6 @@ class MainWindow (QMainWindow):
             # Needed for PyQt4 to prevent warning:
             # QObject::startTimer: QTimer can only be used with threads started with QThread
             self.ui.treeview.setModel(None)
-            # this line prevents a segmentation fault with PySide if game->quit selected
-            self.cube_area.setMouseTracking(False)
         finally:
             QMainWindow.closeEvent(self, event)
         
@@ -419,7 +458,6 @@ class MainWindow (QMainWindow):
             model = Model(sizes, mirror_distance)
             self.gamestate.set_model(model)
             self.cube_area.set_model(model)
-            self.cube_area.update()
         elif key == 'draw.accels':
             self.move_keys = self.get_move_key_dict()
             
@@ -451,8 +489,8 @@ class MainWindow (QMainWindow):
             
     ### GL area handlers ###
     
-    def on_cubearea_end_animation(self, last_move):
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+    def on_cubearea_animation_ending(self, last_move):
+        self.cube_area.set_transformations(self.gamestate.current_cube_state)
         
         if not last_move:
             # go again
@@ -461,10 +499,10 @@ class MainWindow (QMainWindow):
                 move_data = self.gamestate.request_back()
             else:
                 move_data = self.gamestate.request_next()
-            if not self.start_animation(move_data, stop_after=False, animtype=self.animtype):
-                self.update_ui()
-        else:
-            self.update_ui()
+            if self.start_animation(move_data, stop_after=False, animtype=self.animtype):
+                return
+        self.cube_area.animation_end()
+        self.update_ui()
             
     def on_cubearea_request_rotation(self, maxis, mslice, mdir):
         self.gamestate.request_rotation(maxis, mslice, mdir)
@@ -474,11 +512,11 @@ class MainWindow (QMainWindow):
         
     def on_cubearea_request_swap_blocks(self, blockpos, maxis, mslice, mdir):
         self.gamestate.swap_block(blockpos, maxis, mslice, mdir)
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+        self.cube_area.set_transformations(self.gamestate.current_cube_state)
         
     def on_cubearea_request_rotate_block(self, blockpos, rdir):
         self.gamestate.rotate_block(blockpos, rdir)
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+        self.cube_area.set_transformations(self.gamestate.current_cube_state)
         
     def on_cubearea_drop_color(self, blocknum, facesym, colorname):
         if blocknum < 0:
@@ -492,6 +530,9 @@ class MainWindow (QMainWindow):
         colornum = self.gamestate.current_cube_state.get_colornum(blocknum, facesym)
         settings.theme.face[colornum].image = filename
         
+    def on_cubearea_debug_text(self, text):
+        self.ui.label_debug_text.setText(text)
+        
     ###
     
     @Slot(int, int)
@@ -522,12 +563,12 @@ class MainWindow (QMainWindow):
         position = self.gamestate.all_moves.mark_and_extend(moves)
         if position >= 0:
             self.gamestate.do_fast_forward(position)
-            self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+            self.cube_area.set_transformations(self.gamestate.current_cube_state)
             self.update_ui()
             move_data = self.gamestate.request_next()
             self.start_animation(move_data, stop_after=False, animtype=AnimType.NEW)
         elif state is not None:
-            self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+            self.cube_area.set_transformations(self.gamestate.current_cube_state)
             self.update_ui()
         self.cube_area.update()
         
@@ -569,7 +610,8 @@ class MainWindow (QMainWindow):
         self.cube_area.reset_rotation()
     @Slot()
     def on_action_info_activated(self):
-        dialogs.show_about(self)
+        about = dialogs.AboutDialog(self)
+        about.run()
         
     @Slot()
     def on_action_rewind_activated(self):
@@ -581,7 +623,7 @@ class MainWindow (QMainWindow):
             self.cube_area.animate_abort()
         else:
             needs_update = self.gamestate.do_rewind_to_mark()
-            self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+            self.cube_area.set_transformations(self.gamestate.current_cube_state)
             if needs_update:
                 self.cube_area.update()
                 self.update_ui()
@@ -635,7 +677,7 @@ class MainWindow (QMainWindow):
             self.cube_area.animate_abort()
         else:
             self.gamestate.do_fast_forward()
-            self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+            self.cube_area.set_transformations(self.gamestate.current_cube_state)
             self.cube_area.update()
             self.update_ui()
     @Slot()
@@ -661,7 +703,7 @@ class MainWindow (QMainWindow):
         if self.is_animating():
             return
         self.gamestate.invert_moves()
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+        self.cube_area.set_transformations(self.gamestate.current_cube_state)
         self.cube_area.update()
         self.update_ui()
     @Slot()
@@ -669,29 +711,76 @@ class MainWindow (QMainWindow):
         if self.is_animating():
             return
         self.gamestate.normalize_complete_rotations()
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+        self.cube_area.set_transformations(self.gamestate.current_cube_state)
         self.cube_area.update()
         self.update_ui()
         
     ### edit formula handlers ###
     
     def on_action_jump_to_editbar_triggered(self):
-        self.ui.edit_moves.setFocus()
+        self.edit_moves.setFocus()
         
-    @Slot()
     def on_edit_moves_returnPressed(self):
         if self.is_animating():
             self.cube_area.animate_abort(update=False)
-        code = self.ui.edit_moves.text()
-        pos = self.ui.edit_moves.cursorPosition()
+        code = self.edit_moves.text()
+        pos = self.edit_moves.cursorPosition()
         self.gamestate.set_from_formula(code, pos)
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+        self.cube_area.set_transformations(self.gamestate.current_cube_state)
         self.cube_area.update()
         self.update_ui()
         
+    def on_edit_moves_nextword(self):
+        code = self.edit_moves.text()
+        pos = self.edit_moves.cursorPosition()
+        moves, mpos, cpos = self.gamestate.all_moves.converter.parse(code, pos,
+                                        self.gamestate.current_cube_state.model)
+        moves.current_place = mpos
+        if cpos <= pos:
+            moves.advance()
+        code, pos, unused_markup = moves.format(self.gamestate.current_cube_state.model)
+        self.edit_moves.setCursorPosition(pos)
+        
+    def on_edit_moves_prevword(self):
+        code = self.edit_moves.text()
+        pos = self.edit_moves.cursorPosition()
+        moves, mpos, cpos = self.gamestate.all_moves.converter.parse(code, pos,
+                                        self.gamestate.current_cube_state.model)
+        moves.current_place = mpos
+        if cpos >= pos:
+            moves.retard()
+        code, pos, unused_markup = moves.format(self.gamestate.current_cube_state.model)
+        self.edit_moves.setCursorPosition(pos)
+        
+    def on_edit_moves_swapnext(self):
+        code = self.edit_moves.text()
+        pos = self.edit_moves.cursorPosition()
+        moves, mpos, cpos = self.gamestate.all_moves.converter.parse(code, pos,
+                                        self.gamestate.current_cube_state.model)
+        moves.current_place = mpos
+        if pos < cpos:
+            moves.retard()
+        moves.swapnext()
+        code, pos, unused_markup = moves.format(self.gamestate.current_cube_state.model)
+        self.edit_moves.setText(code)
+        self.edit_moves.setCursorPosition(pos)
+        
+    def on_edit_moves_swapprev(self):
+        code = self.edit_moves.text()
+        pos = self.edit_moves.cursorPosition()
+        moves, mpos, cpos = self.gamestate.all_moves.converter.parse(code, pos,
+                                        self.gamestate.current_cube_state.model)
+        moves.current_place = mpos
+        if pos < cpos:
+            moves.retard()
+        moves.swapprev()
+        code, pos, unused_markup = moves.format(self.gamestate.current_cube_state.model)
+        self.edit_moves.setText(code)
+        self.edit_moves.setCursorPosition(pos)
+        
     @Slot()
     def on_button_edit_clear_clicked(self):
-        self.ui.edit_moves.setText('')
+        self.edit_moves.setText('')
         self.on_edit_moves_returnPressed()
         
     @Slot()
@@ -699,18 +788,20 @@ class MainWindow (QMainWindow):
         self.on_edit_moves_returnPressed()
         
     def set_edit_moves(self, code, pos, unused_markup=None):
-        self.ui.edit_moves.setText(code)
-        self.ui.edit_moves.setCursorPosition(pos)
+        self.edit_moves.setText(code)
+        self.edit_moves.setCursorPosition(pos)
         
     def on_action_edit_model_triggered(self):
-        self.cube_area.animate_abort(False)
+        self.cube_area.animate_abort(update=False)
         self.gamestate.do_rewind_to_start()
         if DEBUG_ROTATE:
             self.gamestate.current_cube_state.debug_blocksymbols(allsyms=True)
-        self.cube_area.apply_to_glmodel(self.gamestate.current_cube_state)
+        self.cube_area.set_transformations(self.gamestate.current_cube_state)
         self.cube_area.set_editing_mode(True)
         self.update_ui()
         
+    def on_action_reload_shader_triggered(self):
+        self.cube_area.reload_shader()
         
 class PlaybarState (object):
     def __init__(self, play_actions):
