@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009, 2011-2012  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009, 2011-2013  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,14 +21,15 @@
 # Original copyright and license: 2003, 2004  John Darrington,  GPL3+
 
 # pylint: disable=C0321
-from __future__ import print_function, division, unicode_literals
+
 
 import os
 import re
 
 # pylint: disable=W0614,W0401
-from PySide.QtCore import *
-from PySide.QtGui import *
+from PyQt4.QtCore import *
+from PyQt4.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
+from PyQt4.QtGui import *
 
 from .debug import *
 # pylint: enable=W0614,W0401
@@ -47,10 +48,8 @@ class Dialog (QDialog):
     _instance = None
     _classes = []
     
-    def __init__(self, parent, ui):
+    def __init__(self, parent):
         QDialog.__init__(self, parent)
-        self.ui = ui
-        self.ui.setupUi(self)
         self.accepted.connect(self.on_response)
         self._ignore_changed = False
         self.bound = {}
@@ -70,7 +69,8 @@ class Dialog (QDialog):
         
     @classmethod
     def delete(cls):
-        for cls_ in cls._classes:
+        while cls._classes:
+            cls_ = cls._classes.pop(0)
             # pylint: disable=W0212
             cls_._instance.close()
             cls_._instance.deleteLater()
@@ -78,9 +78,9 @@ class Dialog (QDialog):
             cls_._instance = None
             
     def bind(self, key, widget, getter, setter, signal):
-        if isinstance(getter, basestring): getter = getattr(widget, getter)
-        if isinstance(setter, basestring): setter = getattr(widget, setter)
-        if isinstance(signal, basestring): signal = getattr(widget, signal)
+        if isinstance(getter, str): getter = getattr(widget, getter)
+        if isinstance(setter, str): setter = getattr(widget, setter)
+        if isinstance(signal, str): signal = getattr(widget, signal)
         setter(getattr(settings, key))
         if signal is not None:
             signal.connect(lambda: self.on_widget_changed(key))
@@ -165,7 +165,7 @@ class ShortcutEditor (QLabel):
     key_pressed = Signal()
     
     def __init__(self, parent):
-        QLabel.__init__(self, _(u'Press a key …'), parent)
+        QLabel.__init__(self, _('Press a key …'), parent)
         self.setFocusPolicy(Qt.StrongFocus)
         self.setAutoFillBackground(True)
         self.key = None
@@ -208,12 +208,13 @@ class ShortcutDelegate (QStyledItemDelegate):
         
         
 class PreferencesDialog (Dialog):
-    accum_buffers = False
     sample_buffers = 0
         
     def __init__(self, parent):
-        from .ui.preferences import Ui_DialogPreferences
-        Dialog.__init__(self, parent, Ui_DialogPreferences())
+        Dialog.__init__(self, parent)
+        self.ui = UI(self, 'preferences.ui')
+        from .ui.preferences import retranslate
+        retranslate(self)
         
         self.ui.button_animspeed_reset.setIcon(QIcon.fromTheme('edit-clear'))
         self.ui.button_lighting_reset.setIcon(QIcon.fromTheme('edit-clear'))
@@ -240,13 +241,7 @@ class PreferencesDialog (Dialog):
         self.bind('draw.lighting', self.ui.checkbox_lighting, 'isChecked', 'setChecked', 'toggled')
         self.bind_reset('draw.lighting', self.ui.button_lighting_reset)
         
-        for text in settings.draw.accum_range:
-            self.ui.combobox_antialiasing.addItem(_(text), text)
-        self.ui.combobox_antialiasing.setCurrentIndex(settings.draw.accum)
-        self.bind('draw.accum', self.ui.combobox_antialiasing,
-                                  'currentIndex', 'setCurrentIndex', 'currentIndexChanged')
         def reset_antialiasing():
-            del settings.draw.accum
             del settings.draw.samples
         self.ui.button_antialiasing_reset.clicked.connect(reset_antialiasing)
         for text in settings.draw.samples_range:
@@ -264,6 +259,10 @@ class PreferencesDialog (Dialog):
         self.bind('draw.mirror_distance', self.ui.spinbox_mirror_faces, 'value', 'setValue', 'valueChanged')
         self.bind_reset('draw.mirror_distance', self.ui.button_mirror_faces_reset)
         
+        html = '<html><head/><body><p><span style=" color:#aa0000;">{}</span></p></body></html>'
+        html = html.format(Qt.escape(_('The program needs to be restarted for the changes to take effect.')))
+        self.ui.label_needs_restarted.setText(html)
+        
         # mouse tab
         def set_selection_mode(unused_mode):
             if settings.draw.selection_nick == 'quadrant':
@@ -293,12 +292,13 @@ class PreferencesDialog (Dialog):
         self.ui.button_background_color = None
         self.setTabOrder(self.ui.radiobutton_mosaic, self.button_background_color)
         
+        # Only a single color, no picture or pattern
         self.ui.combobox_image.addItem(_('plain'), '')
         for filename in textures.stock_files:
             pixmap = QPixmap(textures.get_stock_pixbuf(filename))
             icon = QIcon(pixmap)
             self.ui.combobox_image.addItem(icon, '', filename)
-        self.ui.combobox_image.addItem(_(u'select …'), '/')
+        self.ui.combobox_image.addItem(_('select …'), '/')
         
         self.bind_reset_item('theme.face.{}.color', self.ui.button_color_reset)
         self.bind_reset_item('theme.face.{}.image', self.ui.button_image_reset)
@@ -311,7 +311,7 @@ class PreferencesDialog (Dialog):
     def create_face_selector(self):
         liststore_faces = QStandardItemModel()
         face_names = [_('Up'), _('Down'), _('Left'), _('Right'), _('Front'), _('Back')]
-        for i in xrange(6):
+        for i in range(6):
             liststore_faces.appendRow((QStandardItem(i), QStandardItem(face_names[i])))
             filename = settings.theme.face[i].image
             if filename.startswith('/'):
@@ -369,13 +369,8 @@ class PreferencesDialog (Dialog):
         if self._ignore_changed:
             return
         self._ignore_changed = True
-        if key in ['draw.accum', 'draw.samples']:
-            if not self.accum_buffers and settings.draw.accum > 0:
-                visible = True
-            elif self.sample_buffers != 2**settings.draw.samples > 1:
-                visible = True
-            else:
-                visible = False
+        if key == 'draw.samples':
+            visible = (self.sample_buffers != 2**settings.draw.samples > 1)
             self.ui.label_needs_restarted.setVisible(visible)
         elif key == 'draw.accels':
             self.liststore_movekeys.clear()
@@ -516,8 +511,10 @@ class SelectModelDialog (Dialog):
     response_ok = Signal(int, tuple, bool)
     
     def __init__(self, parent):
-        from .ui.model import Ui_DialogSelectModel
-        Dialog.__init__(self, parent, Ui_DialogSelectModel())
+        Dialog.__init__(self, parent)
+        self.ui = UI(self, 'model.ui')
+        from .ui.model import retranslate
+        retranslate(self)
         
         for text in settings.game.type_range:
             self.ui.combobox_model.addItem(_(text), text)
@@ -591,49 +588,164 @@ def linkedtext_to_html(text):
     html = re.sub(r'&lt;\|&gt;', r'</a>', html)
     return re.sub(r'&lt;((?:http:|https:).*?)&gt;', r'&lt;<a href="\1">\1</a>&gt;', html)
     
-def show_about(toplevel):
-    from .ui.about import Ui_AboutDialog
-    dialog = QDialog(toplevel)
-    about = Ui_AboutDialog()
-    about.setupUi(dialog)
-    about.label_appname.setText(_(config.APPNAME))
-    about.label_version.setText(config.VERSION)
-    about.label_copyright.setText(config.COPYRIGHT)
-    html = linkedtext_to_html(config.get_filebug_text())
-    about.label_feedback.setText(html)
-    html = linkedtext_to_html(_(config.TRANSLATION_TEXT))
-    about.label_translation.setText(html)
-    about.text_license_short.hide()
-    about.text_license_full.hide()
-    html = linkedtext_to_html('\n\n'.join((_(config.LICENSE_INFO), _(config.LICENSE_FURTHER))))
-    about.text_license_short.setHtml(html)
-    try:
-        with open(config.LICENSE_FILE) as license_file:
-            text = license_file.read()
-    except EnvironmentError:
-        text = _(config.LICENSE_NOT_FOUND)
-        about.text_license_full.setLineWrapMode(QTextEdit.WidgetWidth)
-    html = linkedtext_to_html(text)
-    about.text_license_full.setHtml(html)
-    about.label_description.setText(_(config.SHORT_DESCRIPTION))
-    about.label_website.setText(
-            '<html><head/><body><p><a href="{}">{}</a></p></body></html>'
-            .format(config.WEBSITE, _('Pybik project website')))
-    about.label_icon.setPixmap(QPixmap(os.path.join(config.PIXMAP_DIR, 'pybik-64')))
-    license_index = about.tab_widget.indexOf(about.tab_license)
-    def tab_widget_currentChanged(index):
-        if index == license_index:
-            about.text_license_short.setVisible(True)
-            about.text_license_full.setVisible(False)
-        QTimer.singleShot(0, dialog.adjustSize)
-    about.tab_widget.currentChanged.connect(tab_widget_currentChanged)
-    def text_license_anchorClicked(url):
+class UI:
+    def __init__(self, toplevel, filename):
+        self._toplevel = toplevel
+        from PyQt4 import uic
+        uic.loadUi(os.path.join(config.UI_DIR, filename), toplevel)
+    def __getattr__(self, attrname):
+        if attrname[0] == '_':
+            return object.__getattr__(self, attrname)
+        else:
+            return self._toplevel.findChild(QObject, attrname)
+            
+        
+class AboutDialog (QDialog):
+    def __init__(self, parent):
+        QDialog.__init__(self, parent)
+        self.about = UI(self, 'about.ui')
+        from .ui.about import retranslate
+        retranslate(self)
+        
+        self.fill_header()
+        self.fill_about_tab()
+        self.fill_feedback_tab()
+        self.fill_translation_tab()
+        self.fill_license_tab()
+        
+        self.index_tab_about = self.about.tab_widget.indexOf(self.about.tab_about)
+        self.index_tab_license = self.about.tab_widget.indexOf(self.about.tab_license)
+        
+        # About tab animation
+        self.scrollbar = self.about.text_translators.verticalScrollBar()
+        self.animation = QPropertyAnimation(self.scrollbar, 'value')
+        self.animation.setLoopCount(-1)
+        
+    def fill_header(self):
+        self.about.label_icon.setPixmap(QPixmap(os.path.join(config.PIXMAP_DIR, 'pybik-64')))
+        self.about.label_appname.setText(_(config.APPNAME))
+        self.about.label_version.setText(config.VERSION)
+        self.about.label_description.setText(_(config.SHORT_DESCRIPTION))
+        
+    def fill_about_tab(self):
+        self.about.label_copyright.setText(config.COPYRIGHT)
+        self.about.label_website.setText(
+                '<html><head/><body><p><a href="{}">{}</a></p></body></html>'
+                .format(config.WEBSITE, _('Pybik project website')))
+        html_template = '''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
+<html><head><meta name="qrichtext" content="1" /><style type="text/css">
+p, li {{ white-space: pre-wrap; }}
+</style></head>
+<body>{}</body></html>'''
+        html_p_template = '<p align="center">{}</p>'
+        html_language_template = '<span style=" font-weight:600;">{}</span>'
+        html_person_template = '<a href="{}">{}</a>'
+        html_p_items = []
+        from .translators import translators
+        try:
+            import icu
+        except ImportError:
+            print('PyICU is not installed')
+            langname_from_code = lambda code, name: name
+        else:
+            def langname_from_code(code, name):
+                iculocale = icu.Locale.createFromName(code)
+                lang = iculocale.getLanguage()
+                if icu.Locale.createFromName(lang).getDisplayName() == lang:
+                    return name
+                return str(iculocale.getDisplayName()) or name
+        import locale
+        sortfunc = lambda v: locale.strxfrm(v[0])
+        def gentranslators():
+            for language, langcode, persons in translators:
+                language = langname_from_code(langcode, language)
+                yield language, persons
+        for language, persons in sorted(gentranslators(), key=sortfunc):
+            language = Qt.escape(language)
+            html_items = [html_language_template.format(language)]
+            for name, link in sorted(persons, key=sortfunc):
+                name = Qt.escape(name)
+                html_items.append(html_person_template.format(link, name))
+            html_p_items.append(html_p_template.format('<br/>'.join(html_items)))
+        html = html_template.format(''.join(html_p_items))
+        self.about.text_translators.setHtml(html)
+        self.about.text_translators.anchorClicked.connect(QDesktopServices.openUrl)
+        self.about.text_translators.viewport().installEventFilter(self)
+        
+    def fill_feedback_tab(self):
+        html = linkedtext_to_html(config.get_filebug_text())
+        self.about.label_feedback.setText(html)
+        
+    def fill_translation_tab(self):
+        html = linkedtext_to_html(_(config.TRANSLATION_TEXT))
+        self.about.label_translation.setText(html)
+        
+    def fill_license_tab(self):
+        self.about.text_license_short.hide()
+        self.about.text_license_full.hide()
+        html = linkedtext_to_html('\n\n'.join((_(config.LICENSE_INFO), _(config.LICENSE_FURTHER))))
+        self.about.text_license_short.setHtml(html)
+        try:
+            with open(config.LICENSE_FILE, 'rt', encoding='utf-8') as license_file:
+                text = license_file.read()
+        except Exception as e:
+            print('Unable to find license text:', e)
+            text = _(config.LICENSE_NOT_FOUND)
+            self.about.text_license_full.setLineWrapMode(QTextEdit.WidgetWidth)
+        html = linkedtext_to_html(text)
+        self.about.text_license_full.setHtml(html)
+        self.about.tab_widget.currentChanged.connect(self._on_tab_widget_currentChanged)
+        self.about.text_license_short.anchorClicked.connect(self._on_text_license_anchorClicked)
+        
+    def update_animation(self):
+        smin = self.scrollbar.minimum()
+        smax = self.scrollbar.maximum()
+        if smax <= smin:
+            return
+        self.animation.setDuration((smax-smin) * 40)
+        self.animation.setKeyValueAt(0., smin)
+        self.animation.setKeyValueAt(0.04, smin)
+        self.animation.setKeyValueAt(0.50, smax)
+        self.animation.setKeyValueAt(0.54, smax)
+        self.animation.setKeyValueAt(1., smin)
+        
+    def showEvent(self, event):
+        if self.animation.state() == QAbstractAnimation.Stopped:
+            self.update_animation()
+            self.scrollbar.hide()
+            self.animation.start()
+            
+    def resizeEvent(self, event):
+        self.update_animation()
+        
+    def eventFilter(self, obj, event):
+        #assert obj == self.about.text_translators.viewport()
+        if event.type() in [QEvent.MouseButtonPress, QEvent.Wheel]:
+            self.animation.pause()
+            self.scrollbar.show()
+        return False
+        
+    def _on_tab_widget_currentChanged(self, index):
+        if index == self.index_tab_about:
+            self.animation.resume()
+            self.scrollbar.hide()
+        else:
+            self.animation.pause()
+            self.scrollbar.show()
+        if index == self.index_tab_license:
+            self.about.text_license_short.setVisible(True)
+            self.about.text_license_full.setVisible(False)
+        QTimer.singleShot(0, self.adjustSize)
+        
+    def _on_text_license_anchorClicked(self, url):
         if url.toString() == 'text:FULL_LICENSE_TEXT':
-            about.text_license_short.setVisible(False)
-            about.text_license_full.setVisible(True)
+            self.about.text_license_short.setVisible(False)
+            self.about.text_license_full.setVisible(True)
         else:
             QDesktopServices.openUrl(url)
-    about.text_license_short.anchorClicked.connect(text_license_anchorClicked)
-    dialog.exec_()
-    dialog.deleteLater()
+            
+    def run(self):
+        self.exec_()
+        self.deleteLater()
+        
 
