@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009, 2011-2012  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009, 2011-2013  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 # Original filename: main.c
 # Original copyright and license: 1998, 2003--2004  John Darrington, GPL3+
 
-from __future__ import print_function, division, unicode_literals
+
 
 import sys, os
 from collections import namedtuple
@@ -82,7 +82,17 @@ def print_version(*unused_args):
     print(config.wrap(config.LICENSE_NOT_FOUND))
     sys.exit(0)
     
+def print_tests(*args):
+    for filename in config.get_testdatafiles():
+        print(filename)
+    sys.exit(0)
+    
+def select_all_tests(value, opts):
+    opts.setdefault('test', []).extend(config.get_testdatafiles())
+    
 debug_level_names = [__a[6:].lower() for __a in debug.__all__ if __a.startswith('DEBUG_')]
+test_arg_names = ['write-y', 'write-yes', 'write-n', 'write-no', 'write-e', 'write-error',
+                  'log-file', 'log-widgets']
 
 arg_info = [
         (None, None, config.LONG_DESCRIPTION.replace('ő', 'o')), #XXX: help2man cannot handle unicode
@@ -93,20 +103,24 @@ arg_info = [
                             'Show help message and exit'),
         (['--version'], print_version,
                             'Show version number and exit'),
-        (['--config-file='], lambda value, opts: opts.setdefault('config-file', value),
+        (['--config-file='], lambda value, opts: opts.update(config_file=value),
                             'Specify the configuration file to use instead of the default location'),
-        (['--defaultconfig'], lambda value, opts: opts.setdefault('defaultconfig', True),
+        (['--defaultconfig'], lambda value, opts: opts.update(defaultconfig=True),
                             'Print default settings to stdout and exit'),
-        (['--pyside'], lambda value, opts: opts.setdefault('pyside', True),
-                            'Use PySide, cannot combined with --pyqt4'),
-        (['--pyqt4'], lambda value, opts: opts.setdefault('pyqt4', True),
-                            'Use PyQt4, cannot be combined with --pyside,'
-                            ' is neither --pyside nor --pyqt4 is given, the application'
-                            ' first imports PySide and if that fails it imports PyQt4'),
-        (['--pure-python'], lambda value, opts: opts.setdefault('pure-python', True),
-                            'Use python module for rendering (very slow)'),
-        (['--debug='], lambda value, opts: opts.setdefault('debug', value),
-                            'Enable debug output, D is a comma-separated list of debug flags:\n{0}'
+        (['--shader='], lambda value, opts: opts.update(shader=value),
+                            'Use a different shader'),
+        (['--list-tests'], print_tests,
+                            'List tests'),
+        (['--test'], select_all_tests,
+                            'Run all tests'),
+        (['--test='], lambda value, opts: opts.setdefault('test', []).append(value),
+                            'Run test T, can be used multiple times'),
+        (['--test-args='], lambda value, opts: opts['test_args'].extend(a for a in value.split(',') if a),
+                            'Comma-separated list of test arguments:\n{}'
+                            .format(' '.join(test_arg_names))),
+        (['--debug='], lambda value, opts: opts.setdefault('debug_flags', []).extend(
+                                            f for f in value.split(',') if f),
+                            'D is a comma-separated list of debug flags:\n{0}'
                             .format(' '.join(debug_level_names))),
         (None, None,  'Qt-Options, for a full list refer to the Qt-Reference,'
                 ' but not all options make sense for this application:'),
@@ -125,8 +139,15 @@ arg_info = [
     
 def parse_args(args):
     arg_functs = {o: f for opts, f, h in arg_info for o in opts or [] if f is not None}
-    opts = {}
-    ui_args = []
+    opts = {
+            'config_file': config.USER_SETTINGS_FILE,
+            'defaultconfig': False,
+            'shader': None,
+            'pure_python': False,
+            'test': [],
+            'test_args': [],
+            'ui_args': [],
+        }
     for arg in args:
         try:
             index = arg.index('=')
@@ -137,113 +158,33 @@ def parse_args(args):
         try:
             func = arg_functs[arg]
         except KeyError:
-            ui_args.append(arg)
+            opts['ui_args'].append(arg)
         else:
             func(value, opts)
             
-    if 'pyside' in opts:
-        if 'pyqt4' in opts:
-            print('You cannot use both --pyside and --pyqt4')
+    for a in opts['test_args']:
+        if a not in test_arg_names:
+            print('unknown test argument:', a)
             sys.exit(1)
-        else:
-            pythonqt = 'pyside'
-    else:
-        if 'pyqt4' in opts:
-            pythonqt = 'pyqt4'
-        else:
-            pythonqt = None
-            
-    debug_flags = opts.get('debug', None)
-    if debug_flags is not None:
-        debug_flags = [f for f in debug_flags.split(',') if f]
-        for d in debug_flags:
-            if d not in debug_level_names:
-                print('unknown debug option:', d)
-                sys.exit(1)
-            
-    Opts = namedtuple('Opts', 'debug_flags pythonqt pure_python ui_args config_file defaultconfig')
-    return Opts(debug_flags, pythonqt, opts.get('pure-python', False), ui_args,
-                opts.get('config-file', config.USER_SETTINGS_FILE), opts.get('defaultconfig', False))
+        
+    for d in opts.get('debug_flags', []):
+        if d not in debug_level_names:
+            print('unknown debug option:', d)
+            sys.exit(1)
+    opts.setdefault('debug_flags', None)
     
-def import_ext(pythonqt, pure_python):
-    if pythonqt != 'pyqt4':
-        debug.debug('Importing PySide')
-        try:
-            import PySide, PySide.QtCore, PySide.QtGui, PySide.QtOpenGL
-        except ImportError as e:
-            debug.debug('  Failed')
-            if pythonqt == 'pyside':
-                print('PySide not found.', e)
-                sys.exit(1)
-            pythonqt = 'pyqt4'
-    if pythonqt == 'pyqt4':
-        debug.debug('Importing PyQt4')
-        try:
-            import sip
-            sip.setapi('QString', 2)
-            sip.setapi('QVariant', 2)
-            import PyQt4
-            import PyQt4.QtCore
-            import PyQt4.QtGui
-            import PyQt4.QtOpenGL
-        except ImportError as e:
-            debug.debug('  Failed')
-            print('Python bindings for Qt4 not found.', e)
-            sys.exit(1)
-        sys.modules[b'PySide'] = PyQt4
-        sys.modules[b'PySide.QtCore'] = PyQt4.QtCore
-        sys.modules[b'PySide.QtGui'] = PyQt4.QtGui
-        sys.modules[b'PySide.QtOpenGL'] = PyQt4.QtOpenGL
-        # Monkey patch QtCore
-        # pylint: disable=E1101
-        PyQt4.QtCore.Signal = PyQt4.QtCore.pyqtSignal
-        PyQt4.QtCore.Slot = PyQt4.QtCore.pyqtSlot
-        PyQt4.QtCore.Qt.CursorShape.CrossCursor = PyQt4.QtCore.Qt.CrossCursor
-        PyQt4.QtCore.Qt.TransformationMode.SmoothTransformation = \
-                        PyQt4.QtCore.Qt.SmoothTransformation
-        # Monkey patch QtOpenGL
-        PyQt4.QtOpenGL.QGLContext.BindOption.LinearFilteringBindOption = \
-                            PyQt4.QtOpenGL.QGLContext.LinearFilteringBindOption
-        PyQt4.QtOpenGL.QGLContext.BindOption.MipmapBindOption = PyQt4.QtOpenGL.QGLContext.MipmapBindOption
-        # pylint: enable=E1101
-        
-    if debug.DEBUG:
-        import PySide, PySide.QtCore, PySide.QtGui, PySide.QtOpenGL # pylint: disable=W0404
-        debug.debug('Qt modules:', PySide.__name__, PySide.QtCore.__name__,
-                                   PySide.QtGui.__name__, PySide.QtOpenGL.__name__)
-        
-    if not pure_python:
-        try:
-            import _gldraw
-            sys.modules[b'pybiklib.gldraw'] = _gldraw
-            sys.modules[b'_gldraw'] = _gldraw
-        except ImportError as e:
-            print(e)
-            print('Unable to import module _gldraw, using slow fallback.')
-            pure_python = True
-    if not pure_python:
-        try:
-            import _glarea
-            sys.modules[b'pybiklib.glarea'] = _glarea
-        except ImportError as e:
-            del sys.modules[b'pybiklib.gldraw']
-            del sys.modules[b'_gldraw']
-            print(e)
-            print('Unable to import module _glarea, using slow fallback.')
-    if pure_python:
-        try:
-            from OpenGL import GL as unused_GL
-        except ImportError as e:
-            print('Python bindings for OpenGL not found.', e)
-            sys.exit(1)
-
-
-def run_app(root_dir, args, config_file):
+    class Opts: pass
+    for k, v in opts.items():
+        setattr(Opts, k, v)
+    return Opts
+    
+def create_app(root_dir, args):
     import gettext
-    from PySide.QtCore import QLocale, QTranslator, QTimer
-    from PySide.QtGui import QApplication, QPalette, QColor
+    from PyQt4.QtCore import QLocale, QTranslator, Qt
+    from PyQt4.QtGui import QApplication, QPalette, QColor
     
     # initialize QApplication
+    QApplication.setAttribute(Qt.AA_X11InitThreads)
     app = QApplication(args)
     args = app.arguments()[1:]
     if args:
@@ -277,26 +218,26 @@ def run_app(root_dir, args, config_file):
         # different root, e.g. /usr/local, source directory
         LOCALEDIR = config.LOCALE_DIR
     t = gettext.translation(config.PACKAGE, LOCALEDIR, languages=[language], fallback=True)
-    t.install(unicode=True, names=['ngettext'])
+    t.install(names=['ngettext'])
+    return app, translator
     
+def create_window(opts):
     # initialize settings
     from .settings import settings
     try:
-        settings.load(config_file)
+        settings.load(opts.config_file)
     except EnvironmentError as e:
         error_message = N_('An error occurred while reading the settings:\n'
                         '{error_message}').format(error_message=e)
         debug.debug(error_message)
         settings.load('')
+        from PyQt4.QtCore import QTimer
         QTimer.singleShot(0, lambda: window.error_dialog(_(error_message)))
         
     # create main window
-    # The application module can be imported only if a QApplication object is created
+    # The application module must be imported after a QApplication object is created
     from .application import MainWindow
-    window = MainWindow()
-    
-    # run the application
-    app.exec_()
+    return MainWindow(opts)
     
     
 def run(root_dir=None):
@@ -308,14 +249,35 @@ def run(root_dir=None):
         print('debug flags:', *opts.debug_flags)
         debug.debug('Qt args:', opts.ui_args)
     
+    try:
+        import sip
+        sip.setapi('QString', 2)
+        sip.setapi('QVariant', 2)
+        import PyQt4.QtCore
+        import PyQt4.QtGui
+        import PyQt4.QtOpenGL
+    except ImportError as e:
+        print('This program needs PyQt4:', e)
+        sys.exit(1)
+    
     if opts.defaultconfig:
+        # settings needs Qt
         from .settings import settings
         settings.load('')
         settings.keystore.dump(sys.stdout, all=True)
         sys.exit(0)
         
-    import_ext(opts.pythonqt, opts.pure_python)
-    
-    run_app(root_dir, opts.ui_args, opts.config_file)
+    app, translator = create_app(root_dir, opts.ui_args)
+    # keep the translator object for the lifetime of the app object, so that the translation works
+    if opts.test:
+        from pybiktest.testrunner import TestRunner
+        for test in TestRunner.wrap(config.TESTDATA_DIR, opts.test, opts.test_args):
+            opts.config_file = test.settings_file
+            window = create_window(opts)
+            test.run(window)
+            app.exec()
+    else:
+        window = create_window(opts)
+        app.exec()
     
 
