@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 
 #  Copyright © 2012-2013  B. Clausius <barcc@gmx.de>
@@ -16,16 +16,18 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function, division, unicode_literals
+
 
 import os
 from math import sin, cos, pi, sqrt
 import re
-import cPickle as pickle
+import pickle as pickle
+from collections import OrderedDict
+import array
 
 import numpy as np
 
-from .debug import DEBUG, DEBUG_NOLABEL, DEBUG_NOBEVEL, DEBUG_INNER
+from .debug import DEBUG, DEBUG_NOLABEL, DEBUG_NOBEVEL, DEBUG_INNER, DEBUG_DRAW
 from .config import MODELS_DIR
 
 N_ = lambda t: t
@@ -112,6 +114,9 @@ class Face (object):
         
     def flip(self):
         self.vertices.reverse()
+        if self.normal is not None:
+            for i in range(len(self.normal)):
+                self.normal[i] = -self.normal[i]
         
         
 class Block (object):
@@ -159,7 +164,7 @@ class Geometry (object):
         for f in self.faces.values():
             f.normal = f.center(self.vertices)
             f.create_scaled(self.vertices, bevel_factor, self.beveled_vertices)
-            f.init_adjacent_faces(self.faces.values())
+            f.init_adjacent_faces(list(self.faces.values()))
             
     def create_permutation(self, matrix):
         m = np.matrix(matrix)
@@ -179,10 +184,10 @@ class Geometry (object):
         
 class EmptyModel (object):
     type = 'Empty'
-    mformat = u'Empty'
-    symbols = b''
-    symbolsI = b''
-    faces = b''
+    mformat = 'Empty'
+    symbols = ''
+    symbolsI = ''
+    faces = ''
     
     sizes = [0, 0, 0]
     size = None
@@ -190,35 +195,21 @@ class EmptyModel (object):
     blocks = []
     
     def __init__(self, *unused_args):
-        # The arrays should never be empty. For empty numpy-arrays it is
-        # not possible to get the C-address in module gldraw (compiled mode).
-        # The fields can only become empty when certain debugging options are active.
-        glvertices = np.array([0], dtype='f')
-        glnormals = np.array([0], dtype='f')
-        glcolors = np.array([0], dtype=np.ubyte)
-        gltexpostiled = np.array([0], dtype='f')
-        gltexposmosaic = np.array([0], dtype='f')
-        self.gldata = [glvertices, glnormals, glcolors, gltexpostiled, gltexposmosaic]
+        pass
         
-    def gl_data(self):
-        return tuple(self.gldata) + ([], 0, 0)
-                
-    def gl_pick_data(self, unused_selection_mode):
-        # pylint: disable=W0201
-        self.gl_pick_vertices = np.array([0], dtype='f')
-        self.gl_pick_colors = np.array([0], dtype=np.ubyte)
-        return self.gl_pick_vertices, self.gl_pick_colors
+    def gl_vertex_data(self, unused_selection_mode, unused_design):
+        return b'', b'', b'', b'', [], 0, [], 0, 0
         
         
 class BrickModel (object):
     type = N_('Brick')
-    mformat = _(u'{0}×{1}×{2}-Brick')
+    mformat = _('{0}×{1}×{2}-Brick')
     symmetry = 180., 180., 180.
     
     axes = [(-1,0,0), (0,-1,0), (0,0,-1)]   # pylint: disable=C0324
-    symbols = b'LDB'
-    symbolsI = b'RUF'
-    faces = b'UDLRFB'
+    symbols = 'LDB'
+    symbolsI = 'RUF'
+    faces = 'UDLRFB'
     epsilon = 0.00001
     
     #### geometry of the cube ####
@@ -237,15 +228,17 @@ class BrickModel (object):
                 vertices=[[-1,-1,-1], [1,-1,-1], [-1, 1,-1], [1, 1,-1],
                           [-1,-1, 1], [1,-1, 1], [-1, 1, 1], [1, 1, 1]],
                 # vertex-indices for unbeveled faces, used for picking
-                faces={b'U': Face(b'U', [2,6,7,3]), b'D': Face(b'D', [4,0,1,5]),
-                       b'L': Face(b'L', [2,0,4,6]), b'R': Face(b'R', [7,5,1,3]),
-                       b'F': Face(b'F', [6,4,5,7]), b'B': Face(b'B', [3,1,0,2])},
+                faces=OrderedDict((
+                        ('U', Face('U', [2,6,7,3])), ('D', Face('D', [4,0,1,5])),
+                        ('L', Face('L', [2,0,4,6])), ('R', Face('R', [7,5,1,3])),
+                        ('F', Face('F', [6,4,5,7])), ('B', Face('B', [3,1,0,2])))),
                 bevel_factor=0.9
             )
     texpos_tiled = [(0, 0),  (0, 1),  (1, 1),  (1, 0)]
-    face_axes = {b'U': (0, 2), b'D': (0, 2),
-                 b'L': (2, 1), b'R': (2, 1),
-                 b'F': (0, 1), b'B': (0, 1)}
+    face_axes = OrderedDict((
+                        ('U', (0, 2)), ('D', (0, 2)),
+                        ('L', (2, 1)), ('R', (2, 1)),
+                        ('F', (0, 1)), ('B', (0, 1))))
     
     def __init__(self, size, mirror_distance):
         self.sizes = self.norm_sizes(size) # tuple of lenght 3
@@ -284,8 +277,8 @@ class BrickModel (object):
                     modeldata = pickle.load(datafile)
                 modeldata_cache = modeldata
                 modeldata = modeldata_cache[(self.type, self.sizes)]
-            self.blocks = [Block(None, None, data=data) for data in modeldata[b'blocks']]
-            self.rotated_position = modeldata[b'rotated_position']
+            self.blocks = [Block(None, None, data=data) for data in modeldata['blocks']]
+            self.rotated_position = modeldata['rotated_position']
             self.gl_never_visible_face = self._gl_never_visible_face_cached
         else:
             nblocks = self.sizes[0] * self.sizes[1] * self.sizes[2]
@@ -303,11 +296,6 @@ class BrickModel (object):
                 
         self.pick_data = [()]  # list of (maxis, mslice, mdir, face, center, block, symbol)
         
-        # List of numpy-arrays
-        # the gldraw module uses C-pointers to the data, so it is essential
-        # that the arrays do not get lost
-        self.gldata = []
-        
     @classmethod
     def norm_sizes(cls, sizes):
         return sizes
@@ -316,7 +304,7 @@ class BrickModel (object):
     def displaystring(cls, size):
         return cls.mformat.format(*size)
         
-    def __unicode__(self):
+    def __str__(self):
         return self.displaystring(self.sizes)
         
     @classmethod
@@ -357,8 +345,8 @@ class BrickModel (object):
                     for axis, sym, angle in zip(self.axes + self.axesI,
                                          self.symbols + self.symbolsI,
                                          self.symmetry + self.symmetry)]
-        self.normal_rotation_symbols[b''] = b''
-        transform = [(b'', np.matrix(np.identity(4)))]
+        self.normal_rotation_symbols[''] = ''
+        transform = [('', np.matrix(np.identity(4)))]
         for sp, p in prim:
             transform.append((sp, p))
         for sm, m in transform:
@@ -398,13 +386,14 @@ class BrickModel (object):
     def get_datafilename(cls, sizes):
         x, y, unused_z = sizes
         if x <= 2:
-            return b'mdata_01-02'
+            return 'mdata_01-02'
         else:
-            return b'mdata_{:02}_{}'.format(x, 0 if x<=5 else y%2 if x<=8 else y%3)
+            return 'mdata_{:02}_{}'.format(x, 0 if x<=5 else y%2 if x<=8 else y%3)
         
     def get_savedata(self):
+        self.calc_data()
         blocks = [block.__dict__ for block in self.blocks]
-        return {b'blocks': blocks, b'rotated_position': self.rotated_position}
+        return {'blocks': blocks, 'rotated_position': self.rotated_position}
             
     def norm_symbol(self, sym):
         try:
@@ -576,19 +565,22 @@ class BrickModel (object):
             f = face.scaled
             if self.gl_never_visible_face(block, f.vindices):
                 continue
-            for i, vi in enumerate(f.vindices):
+            for i in [0, 1, 2, 0, 2, 3]:
+                vi = f.vindices[i]
                 v = self.geom.beveled_vertices[vi]
                 texpos_tiled = self.texpos_tiled[i]
                 texpos_mosaic = self.texpos_tiled_to_mosaic(block, face, texpos_tiled)
                 yield (v, face.normal, faceno, texpos_tiled, texpos_mosaic)
             if self.mirror_distance is not None and visible:
                 f = f.translated(self.geom.beveled_vertices, face.normal, self.mirror_distance)
+                f.normal = face.normal[:]
                 f.flip()
-                for i, v in enumerate(f.vertices):
+                for i in [0, 1, 2, 0, 2, 3]:
+                    v = f.vertices[i]
                     tptx, tpty = self.texpos_tiled[i]
                     texpos_tiled = 1. - tptx, tpty
                     texpos_mosaic = self.texpos_tiled_to_mosaic(block, face, texpos_tiled)
-                    yield (v, face.normal, faceno, texpos_tiled, texpos_mosaic)
+                    yield (v, f.normal, faceno, texpos_tiled, texpos_mosaic)
                     
     def gl_beveled_quads(self, block):
         '''For every edge create a face'''
@@ -613,11 +605,12 @@ class BrickModel (object):
                 bvi4 = f2.scaled.vindices[f2.vindices.index(vi2)]
                 if self.gl_never_visible_face(block, [bvi1, bvi2, bvi3, bvi4]):
                     continue
-                n = Face('', [bvi1, bvi2, bvi3, bvi4]).center(self.geom.beveled_vertices)
-                yield self.geom.beveled_vertices[bvi1], n
-                yield self.geom.beveled_vertices[bvi2], n
-                yield self.geom.beveled_vertices[bvi3], n
-                yield self.geom.beveled_vertices[bvi4], n
+                yield self.geom.beveled_vertices[bvi1], f.normal
+                yield self.geom.beveled_vertices[bvi2], f.normal
+                yield self.geom.beveled_vertices[bvi3], f2.normal
+                yield self.geom.beveled_vertices[bvi1], f.normal
+                yield self.geom.beveled_vertices[bvi3], f2.normal
+                yield self.geom.beveled_vertices[bvi4], f2.normal
                 
     def gl_beveled_triangles(self, block):
         '''For every corner create a face'''
@@ -630,6 +623,7 @@ class BrickModel (object):
                 return
         for vi in range(len(self.geom.vertices)):
             bf = [] # one beveled face for each vertex
+            bn = []
             for face_first in self.geom.faces.values():
                 if vi in face_first.vindices:
                     # we now have one adjacent face, this code should be reached if the model is valid
@@ -642,6 +636,7 @@ class BrickModel (object):
                 ivi = face.vindices.index(vi)
                 bvi = face.scaled.vindices[ivi]
                 bf.insert(0, bvi) # we need counterclockwise order
+                bn.insert(0, face.normal)
                 symbol = face.faces[ivi]
                 face = self.geom.faces[symbol]
                 ## we now have the clockwise next face
@@ -649,8 +644,7 @@ class BrickModel (object):
                     break
             if self.gl_never_visible_face(block, bf):
                 continue
-            n = Face('', bf).center(self.geom.beveled_vertices)
-            for vi in bf:
+            for vi, n in zip(bf, bn):
                 yield self.geom.beveled_vertices[vi], n
             
     def gl_pick_triangles(self, selection_mode):
@@ -742,14 +736,14 @@ class BrickModel (object):
                         if self.mirror_distance is not None:
                             yield color, vct; yield color, edget[1]; yield color, edget[0]
                     
-    def gl_data(self):
+    def gl_vertex_data(self, selection_mode, design):
         vertices = []
         normals = []
         colors = []
-        texpost = []
-        texposm = []
-        # list of (labelinfo, bevelqinfo, beveltinfo)
-        blockinfos = [[[], 0, 0] for unused_b in self.blocks]
+        texpos = []
+        # list of [count vertices, face index]
+        labelinfos = [None for unused_b in self.blocks]
+        cnts_block = [0 for unused_b in self.blocks]
         
         # label vertices
         for iblock, block in enumerate(self.blocks):
@@ -759,8 +753,8 @@ class BrickModel (object):
             for v, n, f, tpt, tpm in self.gl_label_quads(block, True):
                 vertices += v
                 normals += n
-                texpost += tpt
-                texposm += tpm
+                colors += design.label_colors[f]
+                texpos += tpm if design.imagemodes[f] else tpt
                 if last_faceno != f:
                     cnt = 1
                     facesinfo.append((cnt, f))
@@ -768,52 +762,80 @@ class BrickModel (object):
                 else:
                     cnt += 1
                     facesinfo[-1] = (cnt, f)
-            blockinfos[iblock][0][:] = facesinfo
-        # beveled edges
-        idx_vbevelq = len(vertices)
+            labelinfos[iblock] = facesinfo
+            
+        # bevel, hidden labels
+        idx_block = len(vertices) // 3
         for iblock, block in enumerate(self.blocks):
             cnt = 0
             for v, n in self.gl_beveled_quads(block):
                 vertices += v
                 normals += n
+                colors += design.bevel_color
                 cnt += 1
             for v, n, f, tpt, tpm in self.gl_label_quads(block, False):
                 vertices += v
                 normals += n
+                colors += design.bevel_color
                 cnt += 1
-            blockinfos[iblock][1] = cnt
-        # beveled corners
-        idx_vbevelt = len(vertices)
-        for iblock, block in enumerate(self.blocks):
-            cnt = 0
             for v, n in self.gl_beveled_triangles(block):
                 vertices += v
                 normals += n
+                colors += design.bevel_color
                 cnt += 1
-            blockinfos[iblock][2] = cnt
+            cnts_block[iblock] = cnt
         
-        # The arrays should never be empty. For empty numpy-arrays it is
-        # not possible to get the C-address in module gldraw (compiled mode).
-        # The fields can only become empty when certain debugging options are active.
-        glvertices = np.array(vertices or [0], dtype='f')
-        glnormals = np.array(normals or [0], dtype='f')
-        glcolors = np.array(colors or [0], dtype=np.ubyte)
-        gltexpostiled = np.array(texpost or [0], dtype='f')
-        gltexposmosaic = np.array(texposm or [0], dtype='f')
-        self.gldata.append(glvertices)
-        self.gldata.append(glnormals)
-        self.gldata.append(glcolors)
-        self.gldata.append(gltexpostiled)
-        self.gldata.append(gltexposmosaic)
+        pickvertices, pickcolors = self.gl_pick_data(selection_mode)
+        idx_pick = len(vertices) // 3
+        cnt_pick = len(pickvertices) // 3
+        
+        vertices += pickvertices
+        colors += pickcolors
+        
+        idx_debug = len(vertices) // 3
+        if DEBUG_DRAW:
+            x = float(-self.sizes[0])
+            y = float(-self.sizes[1])
+            z = float(-self.sizes[2])
+            # axes
+            vertices += [x-1, y-1, z,  -x, y-1, z]
+            colors += [255, 0, 0,  255, 0, 0]
+            vertices += [x-1, y-1, z,  x-1, -y, z]
+            colors += [0, 255, 0,  0, 255, 0]
+            vertices += [x-1, y-1, z,  x-1, y-1, -z]
+            colors += [0, 0, 255,  0, 0, 255]
+            # selection (modelview)
+            vertices += [0, 0, 0,  1, 1, 1]
+            colors += [255, 255, 255,  255, 255, 255]
+            # selection (viewport)
+            vertices += [0, 0, 0,  1, 1, 0]
+            colors += [255, 255, 0,  255, 0, 0]
+            
         if DEBUG:
-            assert idx_vbevelq * 2 == len(texpost) * 3 == len(texposm) * 3
-            assert sum([cnt for _bi in blockinfos for cnt, unused_faceno in _bi[0]]) * 3 == idx_vbevelq
+            assert sum([cnt for _bi in labelinfos for cnt, unused_faceno in _bi]) == idx_block
         
-        return (glvertices, glnormals, glcolors, gltexpostiled, gltexposmosaic,
-                    blockinfos,
-                    idx_vbevelq, idx_vbevelt
+        return (    array.array('f', vertices).tobytes(),
+                    array.array('f', normals).tobytes(),
+                    array.array('B', colors).tobytes(),
+                    array.array('f', texpos).tobytes(),
+                    labelinfos,
+                    idx_block, cnts_block,
+                    idx_pick, cnt_pick,
+                    idx_debug,
                )
                 
+    def calc_data(self):
+        '''Function to verify that Block.visible_glfaces lists are initialized.'''
+        for iblock, block in enumerate(self.blocks):
+            for v, n, f, tpt, tpm in self.gl_label_quads(block, True):
+                pass
+            for v, n in self.gl_beveled_quads(block):
+                pass
+            for v, n, f, tpt, tpm in self.gl_label_quads(block, False):
+                pass
+            for v, n in self.gl_beveled_triangles(block):
+                pass
+                
     def gl_pick_data(self, selection_mode):
         # Pick TRIANGLES vertices
         vertices = []
@@ -822,18 +844,13 @@ class BrickModel (object):
             vertices += v
             color = [(col>>4) & 0xf0, (col) & 0xf0, (col<<4) & 0xf0]
             colors += color
-        glvertices = np.array(vertices or [0], dtype='f')
-        glcolors = np.array(colors or [0], dtype=np.ubyte)
-        # pylint: disable=W0201
-        self.gl_pick_vertices = glvertices
-        self.gl_pick_colors = glcolors
-        assert len(glvertices) == len(glcolors)
-        return glvertices, glcolors
+        assert len(vertices) == len(colors)
+        return vertices, colors
         
         
 class TowerModel (BrickModel):
     type = N_('Tower')
-    mformat = _(u'{0}×{1}-Tower')
+    mformat = _('{0}×{1}-Tower')
     symmetry = 180., 90., 180.
     
     def __init__(self, size, mirror_distance):
@@ -848,7 +865,7 @@ class TowerModel (BrickModel):
         
 class CubeModel (BrickModel):
     type = N_('Cube')
-    mformat = _(u'{0}×{0}×{0}-Cube')
+    mformat = _('{0}×{0}×{0}-Cube')
     symmetry = 90., 90., 90.
     
     def __init__(self, size, mirror_distance):
