@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009, 2011-2012  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009, 2011-2013  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 # Original filename: guile-hooks.c
 # Original copyright and license: 2004  Dale Mellor, GPL3+
 
-from __future__ import print_function, division, unicode_literals
+
 
 import os
 import sys
@@ -52,8 +52,8 @@ class PluginState (object):
     def add_flubrd(self, code):
         #TODO: here flubrd should be used, not the one in all_moves,
         # the converter in all_moves may be configurable in the future
-        for move_data, unused_pos in self.moves.parse_iter(code, len(code), self.current_state.model):
-            self.current_state.rotate_slice(move_data)
+        for res in self.moves.parse_iter(code, len(code), self.current_state.model):
+            self.current_state.rotate_slice(res[0])
             
     def random(self, count=-1):
         self.current_state.set_solved()
@@ -108,7 +108,7 @@ class PluginHelper (object):
     def call(self, app, index):
         plugin_state = PluginState(app)
         all_models = []
-        for models, func in self.scripts.values()[index]:
+        for models, func in list(self.scripts.values())[index]:
             if plugin_state.test_model(models, show_error=False):
                 func(plugin_state)
                 break
@@ -123,11 +123,11 @@ class PluginHelper (object):
             return
         debug('Loading Plugins from', dirname)
         sys.path.insert(0, dirname)
-        for filename in sorted(os.listdir(dirname), key=unicode.lower):
+        for filename in sorted(os.listdir(dirname), key=str.lower):
             unused_name, ext = os.path.splitext(filename)
             if ext != '.algorithm':
                 continue
-            debug('  algorithm:', filename)
+            if DEBUG_ALG: debug('  algorithm:', filename)
             try:
                 self.load_text_plugin(os.path.join(dirname, filename))
             except Exception as e:      # pylint: disable=W0703
@@ -141,12 +141,12 @@ class PluginHelper (object):
         del self.error_messages[:]
         self.load_plugins_from_directory(config.SCRIPT_DIR)
         self.load_plugins_from_directory(config.USER_SCRIPT_DIR)
-        debug('  found', len(self.scripts))
+        if DEBUG_ALG: debug('found', len(self.scripts))
         return [(path, i) for i, path in enumerate(self.scripts.keys())]
         
     @staticmethod
     def parse_text_plugin(filename):
-        with open(filename) as fp:
+        with open(filename, 'rt', encoding='utf-8') as fp:
             lines = fp.readlines()
         paras = []
         
@@ -154,7 +154,7 @@ class PluginHelper (object):
         para = {}
         key = None
         for line in lines:
-            line = unicode(line, 'utf-8').rstrip()
+            line = line.rstrip()
             if not line:
                 # end para
                 if para:
@@ -216,7 +216,7 @@ class PluginHelper (object):
         for para in paras:
             value = para.get('Path', ())
             if len(value) != 1 or not value[0]:
-                debug('    skip Path:', value)
+                if DEBUG_ALG: debug('    skip Path:', value)
                 para.clear()
                 continue
             def split_path(value):
@@ -227,10 +227,10 @@ class PluginHelper (object):
                 return sep, tuple(v for v in value if v)
             sep, value = split_path(value[0])
             if not value:
-                debug('    skip Path:', value)
+                if DEBUG_ALG: debug('    skip Path:', value)
                 para.clear()
                 continue
-            debug('    Path:', value)
+            if DEBUG_ALG: debug('    Path:', value)
             para['Path'] = sep, value
             
             value = para.get('Depends', [])
@@ -364,7 +364,7 @@ class ScriptFactory (object):
         
     def test_basic_condition(self, cube, position, condition):
         assert len(position) == len(condition)
-        for i in xrange(len(position)):
+        for i in range(len(position)):
             if not self.test_face(cube, position, condition, i):
                 return False
         return True
