@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #-*- coding:utf-8 -*-
 
 #    Pybik -- A 3 dimensional magic cube game.
-#    Copyright © 2009, 2011-2012  B. Clausius <barcc@gmx.de>
+#    Copyright © 2009, 2011-2013  B. Clausius <barcc@gmx.de>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from __future__ import print_function, division
-
 import sys, os, re
 
 class Py2pyxParseError (Exception): pass
@@ -42,49 +40,42 @@ def main(argv):
     
 def create_pyx(src_path, dst_path, pxd_path):
     errors = 0
-    openw = lambda path: open(path, 'w') if path else sys.stdout
-    remove_lines = 0
+    openr = lambda path: open(path, 'rt', encoding='utf-8')
+    openw = lambda path: open(path, 'wt', encoding='utf-8') if path else sys.stdout
+    edit_lines = None
     
-    copy = lambda lineno, match: (match.string, None, 0)
-    add = lambda lineno, match: (match.expand(r'\1\4 \2\n'), None, 0)
+    copy = lambda lineno, match: (match.string, None, None)
+    add = lambda lineno, match: (match.expand(r'\1\4 \2\n'), None, None)
     def remove(unused_lineno, match):
         count = match.group(4)
-        if count == '':
-            count = 1
-        else:
-            count = int(count)
-        return match.expand(r'\1#px \2\4\n'), None, count
-    replace = lambda lineno, match: (match.expand(r'\1\4 \2\n'), None, 1)
-    comment = lambda lineno, match: (match.expand(r'\1\2\4\n'), None, 0)
-    pxd = lambda lineno, match: (match.expand(r'\1#px \2\4\n'),
-                                 match.expand(r'\1\4') + '    #line {}\n'.format(lineno),
-                                 0)
-    add_pxd = lambda lineno, match: (match.expand(r'\1\4 \2\n'),
-                                     match.expand(r'\1\4\n') + '    #line {}\n'.format(lineno),
-                                     0)
+        count = 1 if count == '' else int(count)
+        return match.expand(r'\1#px \2\4\n'), None, [count, r'\1#px \2\4\n', None]
+    replace = lambda lineno, match: (match.expand(r'\1\4 \2\n'), None, [1, r'\1#px \2\4\n', None])
+    comment = lambda lineno, match: (match.expand(r'\1\2\4\n'), None, None)
     def repl_pxd(lineno, match):
-        if match.group(6):
-            return (match.expand(r'\1\4 \2\n'),
-                    match.expand(r'\1\5') + '    #line {}\n'.format(lineno),
-                    1)
-        else:
-            return (match.expand(r'\1#px \2\4\n'),
-                    match.expand(r'\1\4') + '    #line {}\n'.format(lineno),
-                    1)
+        return (match.expand(r'\1\4 \2\n'),
+                match.expand(r'\1\4').rstrip(':') + '    #line {}\n'.format(lineno),
+                [1, r'\1#px \2\4\n', None])
+    def move_pxd(unused_lineno, match):
+        count = match.group(4)
+        count = 1 if count == '' else int(count)
+        return match.expand(r'\1#px \2\4\n'), None, [count, r'\1#px \2\4\n', r'\1\4\n']
     fdict = {None:copy, '+':add, '-':remove, '/':replace, ' ':comment,
-             'd ':pxd, 'd+':add_pxd, 'd/':repl_pxd}
+             #'d ':pxd, 'd+':add_pxd,
+             'd/':repl_pxd, 'd=':move_pxd}
     
-    with open(src_path,'r') as srcf, openw(dst_path) as dstf, openw(pxd_path) as pxdf:
+    with openr(src_path) as srcf, openw(dst_path) as dstf, openw(pxd_path) as pxdf:
         pxdf.write('#file: {}\n\n'.format(src_path))
         for lineno, line in enumerate(srcf):
             try:
-                match = re.match(r'^( *)(#px(d |d\+|d/|.)|)((.*?)(:?))\n$', line)
-                if remove_lines > 0:
-                    pyxline = match.expand(r'\1#px \2\4\n')
-                    pxdline = None
-                    remove_lines -= 1
+                match = re.match(r'^( *)(#px(d.|.)|)(.*?)\n$', line)
+                if edit_lines is not None and edit_lines[0] > 0:
+                    cnt_edit_lines, edit_re_pyxline, edit_re_pxdline = edit_lines
+                    pyxline = edit_re_pyxline and match.expand(edit_re_pyxline)
+                    pxdline = edit_re_pxdline and (match.expand(edit_re_pxdline).rstrip() + '    #line {}\n'.format(lineno))
+                    edit_lines[0] -= 1
                     continue
-                pyxline, pxdline, remove_lines = fdict[match.group(3)](lineno, match)
+                pyxline, pxdline, edit_lines = fdict[match.group(3)](lineno, match)
             except Exception as e:
                 print('%s:%d:'%(src_path, lineno), e, file=sys.stderr)
                 print('  Invalid line:', line, file=sys.stderr, end='')
